from enum import IntEnum


class SystemRole(IntEnum):
    R_ADM_START = 1
    R_ADM_CONFIG_SYSTEM = 2
    R_ADM_CONFIG_ARCHIVE = 3
    R_ADM_CONFIG_SECURITY = 4
    R_ADM_CONFIG_WMODULE = 5
    R_ADM_CONFIG_DMODULE = 6
    R_ADM_START_LICEDIT = 7
    R_ADM_START_AUTOACTION = 8
    R_ADM_EDIT_AUTOACTION = 9
    R_ADM_VIEW_PROTOCOLS = 10
    R_ASE_START_PROGRAM = 11
    R_ASE_EDIT_OBJDEF = 12
    R_ASE_REBUILD_DB = 13
    R_AAU_START_PROGRAM = 14
    R_AIX_START_PROGRAM = 15
    R_AIX_CONFIG_PROGRAM = 16
    R_CLNT_STORE_SETTINGS = 17
    R_DMS_SUPERVISOR = 18
    R_WFCLNT_USE = 19
    R_WFADM_START = 20
    R_WFEDIT_START = 21
    R_WFEDIT_ORG_EDIT = 22
    R_WFEDIT_MODEL_EDIT = 23
    R_WFIMP_START = 24
    R_WFTEST_START = 25
    R_WFSCRIPT_START = 26
    R_SRV_ADMIN = 27
    R_CLNT_OBJHIST = 28
    R_CLNT_EXPMOD = 29
    R_CLNT_EXPORT = 30
    R_CLNT_NOTES = 31
    R_CLNT_RESTORE_OBJECT = 32
    R_CLNT_TRAY = 33
    R_CLNT_EXTREQ_CFG = 34
    R_CLNT_SENDEXT = 35
    R_CLNT_START = 36
    R_CLNT_SYSTRAY = 37
    R_CLNT_EXTREQ_WORK = 38
    R_CLNT_ABO_ADMIN = 39
    R_CLNT_DELETETRAY = 40
    R_WEBEDIT_START = 41
    R_WEBEDIT_TRANSFER = 42
    R_WEBADM_START = 43
    R_CLNT_OBJINFO = 44
    R_CLNT_DEBUGEVENTS = 45
    R_CLNT_REL_SHOW = 46
    R_CLNT_VIS_CREATE = 47
    R_CLNT_VIS_ADMIN = 48
    R_CLNT_VIS_PUBLIC = 49
    R_ADM_CHG_OBJRIGHTS = 50
    R_CLNT_ARC_BAR = 51
    R_CLNT_ICON_BAR = 52
    R_CLNT_ARC_DELDOCS = 53
    R_ACTVERSYS_START = 54
    R_CLNT_REVISIT_ADMIN = 55
    R_CLNT_CFG_OBJHIST = 56
    R_CLNT_MULTI_MODIFICATION = 57
    R_WFEDIT_ATTENDANCE_EDIT = 58
    R_CLNT_ICON_ADD = 59
    R_CLNT_ICON_DEL = 60
    R_CLNT_WFPROXY = 61
    R_WFCLNT_ADMINISTRATE = 62
    R_CLNT_MOVE_OBJECTS = 63
    R_CLNT_ANN_EDIT = 64
    R_ADM_CONFIG_LOCALSECURITY = 65
    R_DMS_LOCALADM_EDIT = 66
    R_CLNT_WFROUTELIST = 67
    R_CLNT_WFPRIVATEROUTELIST = 68
    R_CLNT_WFPUBLICROUTELIST = 69
    R_WEBCLNT_START = 70
    R_CLNT_VIEWNOTES = 71
    R_SRV_SWITCH_CONTEXT = 72
    R_CLNT_VAR_SHOWALWAYS = 73
    R_CLNT_PRN_DOCS = 74
    R_CLNT_ARC_CHANGESTATE = 75
    R_CLNT_SHOW_MOBFAV = 76
    R_CLNT_ADJUST_RIBBONBAR = 77
    R_CLNT_EXECUTE_INQUIRY = 78
    R_CLNT_ACROSS_CABINET = 79
    R_CLNT_CONFIG_PROGRAMS = 80
    R_CLNT_VIEW_ANNOTATION = 81
    R_CLNT_ANNOTATION = 82
    R_CLNT_CREATE_REFERENCE = 83
    R_CLNT_CREATE_REFERENCE_DOCS = 84
    R_CLNT_SHARE_DOCS = 85
    R_CLNT_MANAGE_SHARED_DOCS = 86
    R_AIX_EDIT_PROGRAM = 87
    R_SRV_ADO_JOBS = 88
    R_CLNT_CLNTS = 89
    R_CLNT_WFVIEWPROXY = 90
    R_CLNT_REVISIT_FOROTHERS = 91
    R_CLNT_ABO_FOROTHERS = 92
    R_CLNT_PUBLICDESKTOP_ADMIN = 93
    R_CLNT_SCAN_DOCS = 94
    R_CLNT_SHOW_REGOBJCOUNT = 95
    R_ASE_DEF_VISIBILITIES = 96
    R_CLNT_EXPORT_INDEX_DATA = 97
    T_CLNT_REVERT_CHECKOUT = 98


import base64
from ecmind_blue_client_manage.system_role import SystemRole
from typing import List
from ecmind_blue_client import Client, Job, Param, ParamTypes
from XmlElement import XmlElement


def get_users(client:Client) -> dict:
    user_list_result = client.execute(Job('mng.GetUserList', Flags=0))

    if not user_list_result.return_code == 0:
        raise RuntimeError(user_list_result.error_message)

    users_element = XmlElement.from_string(user_list_result.values['UserList']).find('Users')
    users_element.find('User').flag_as_list = True
    result = {}
    for user_entry in users_element.to_dict()['User']:
        result[user_entry['@benutzer']] = {
            'id': user_entry['@id'],
            'login': user_entry['@benutzer'],
            'name': user_entry['@name'],
            'guid': user_entry['@osguid'],
            'mail': user_entry['@osemail'],
            'locked': True if user_entry['@locked'] == 1 else False,
            'profile': user_entry['@profil'],
        }
    return result


def get_user_ids(client:Client) -> dict:
    return { u['id']: u for u in get_users(client).values() }


def get_user_guids(client:Client) -> dict:
    return { u['guid']: u for u in get_users(client).values() }


def get_sessions(client: Client) -> list[dict[str, str]]:
    job = Job('krn.SessionEnumDB', Flags=0)
    
    result_get_sessions = client.execute(job)
    if not result_get_sessions.return_code == 0:
        raise RuntimeError(result_get_sessions.error_message)

    sessions = []
    
    for session_data_encoded in result_get_sessions.values['Sessions'].split(';'):
        if len(session_data_encoded) == 0:
            break

        session_data = base64.b64decode(session_data_encoded.encode('utf-8')).decode('utf-8').split('\x00')
        session = { info: session_data[i] for i, info in enumerate(result_get_sessions.values['SessionInfoType'].split(";")) }

        sessions.append(session)

    return sessions


def get_system_roles(client:Client, login:str) -> List[SystemRole]:
    job = Job('mng.GetUserRoles')
    job.append(Param('Flags', ParamTypes.INTEGER, 0))
    job.append(Param('$$$SwitchContextUserName$$$', ParamTypes.STRING, login))
    result_user_roles = client.execute(job)

    if result_user_roles.return_code != 0:
        raise RuntimeError(result_user_roles.error_message)

    return [ 
        SystemRole(int(r))
        for r 
        in str(result_user_roles.values['Result']).strip(';').split(';') 
        if len(r)
    ]

def get_user_groups_by_guid(client:Client, guid:str) -> List[str]:
    job = Job('mng.GetUserGroups', Flags=0, OutputUnicode=1, UserGUID=guid)
    get_user_groups_result = client.execute(job)
    if get_user_groups_result.return_code != 0:
        raise RuntimeError(get_user_groups_result.error_message)

    groups = XmlElement.from_string(get_user_groups_result.values['utfGroupList'])['Groups'][0]
    first_group = groups.find('Group')
    if first_group == None:
        raise RuntimeError('No groups found.')
    first_group.flag_as_list = True

    result = {}
    for group_entry in groups.to_dict(recognize_bool=False)['Group']:
        result[group_entry['@name']] = {
            'id': group_entry['@id'],
            'name': group_entry['@name'],
            'description': group_entry['@description'],
            'guid': group_entry['@osguid'],
            'profil': True if group_entry['@profil'] == 1 else False,
        }
    return result

def get_user_groups_by_login(client:Client, login:str) -> List[str]:
    '''This is a relatively inefficient comfort function using get_users() beforehand get_user_group_by_guid().'''
    users = get_users(client)
    guid = users[login]['guid']
    return get_user_groups_by_guid(client, guid)


import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='ecmind_blue_client_manage',
    version='0.0.5',
    author='Roland Koller, Ulrich Wohlfeil',
    author_email='info@ecmind.ch',
    description='Helper modules for the `ecmind_blue_client` to ease the work with management APIs.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://gitlab.ecmind.ch/open/ecmind_blue_client_manage',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.9',
    install_requires=[
        'ecmind_blue_client>=0.3.7',
        'XmlElement>=0.3.0'
    ],
    extras_require = { }
)